﻿using System;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Drawing;

using Esri.ArcGISRuntime.Data;
using Esri.ArcGISRuntime.Geometry;
using Esri.ArcGISRuntime.Mapping;
using Esri.ArcGISRuntime.Symbology;
using Esri.ArcGISRuntime.UI;

namespace FlickBugDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private GraphicsOverlay sketchOverlay;

        public MainWindow()
        {
            this.InitializeComponent();

            this.Initialize();
        }

        private void Initialize()
        {
            // Create a light gray canvas map
            var myMap = new Map(SpatialReferences.WebMercator);

            // Create graphics overlay to display sketch geometry
            this.sketchOverlay = new GraphicsOverlay();
            this.MyMapView.GraphicsOverlays.Add(this.sketchOverlay);

            // Assign the map to the MapView
            this.MyMapView.Map = myMap;
            this.MyMapView.InteractionOptions = new MapViewInteractionOptions
            {
                IsFlickEnabled = false
            };

            // Fill the combo box with choices for the sketch modes (shapes)
            this.SketchModeComboBox.ItemsSource = System.Enum.GetValues(typeof(SketchCreationMode));
            this.SketchModeComboBox.SelectedIndex = 0;

            // Set the sketch editor configuration to allow vertex editing, resizing, and moving
            var config = this.MyMapView.SketchEditor.EditConfiguration;
            config.AllowVertexEditing = true;
            config.ResizeMode = SketchResizeMode.Uniform;
            config.AllowMove = true;

            // Set the sketch editor as the page's data context
           this.DataContext = this.MyMapView.SketchEditor;
        }

        private Graphic CreateGraphic(Esri.ArcGISRuntime.Geometry.Geometry geometry)
        {
            // Create a graphic to display the specified geometry
            Symbol symbol = null;
            switch (geometry.GeometryType)
            {
                // Symbolize with a fill symbol
                case GeometryType.Envelope:
                case GeometryType.Polygon:
                    {
                        symbol = new SimpleFillSymbol()
                        {
                            Color = Color.Red,
                            Style = SimpleFillSymbolStyle.Solid,
                        };
                        break;
                    }
                // Symbolize with a line symbol
                case GeometryType.Polyline:
                    {
                        symbol = new SimpleLineSymbol()
                        {
                            Color = Color.Red,
                            Style = SimpleLineSymbolStyle.Solid,
                            Width = 5d
                        };
                        break;
                    }
                // Symbolize with a marker symbol
                case GeometryType.Point:
                case GeometryType.Multipoint:
                    {

                        symbol = new SimpleMarkerSymbol()
                        {
                            Color = Color.Red,
                            Style = SimpleMarkerSymbolStyle.Circle,
                            Size = 15d
                        };
                        break;
                    }
            }

            // pass back a new graphic with the appropriate symbol
            return new Graphic(geometry, symbol);
        }

        private async Task<Graphic> GetGraphicAsync()
        {
            // Wait for the user to click a location on the map
            var mapPoint = (MapPoint)await this.MyMapView.SketchEditor.StartAsync(SketchCreationMode.Point, false).ConfigureAwait(true);

            // Convert the map point to a screen point
            var screenCoordinate = this.MyMapView.LocationToScreen(mapPoint);

            // Identify graphics in the graphics overlay using the point
            var results = await this.MyMapView.IdentifyGraphicsOverlaysAsync(screenCoordinate, 2, false).ConfigureAwait(true);

            // If results were found, get the first graphic
            Graphic graphic = null;
            var idResult = results.FirstOrDefault();
            if (idResult != null && idResult.Graphics.Count > 0)
            {
                graphic = idResult.Graphics.FirstOrDefault();
            }

            // Return the graphic (or null if none were found)
            return graphic;
        }

        private async void DrawButtonClick(object sender, RoutedEventArgs e)
        {
            try
            {
                // Let the user draw on the map view using the chosen sketch mode
                var creationMode = (SketchCreationMode)this.SketchModeComboBox.SelectedItem;
                var geometry = await this.MyMapView.SketchEditor.StartAsync(creationMode, true).ConfigureAwait(true);

                // Create and add a graphic from the geometry the user drew
                var graphic = this.CreateGraphic(geometry);
                this.sketchOverlay.Graphics.Add(graphic);

                // Enable/disable the clear and edit buttons according to whether or not graphics exist in the overlay
                this.ClearButton.IsEnabled = this.sketchOverlay.Graphics.Count > 0;
                this.EditButton.IsEnabled = this.sketchOverlay.Graphics.Count > 0;
            }
            catch (TaskCanceledException)
            {
                // Ignore ... let the user cancel drawing
            }
            catch (Exception ex)
            {
                // Report exceptions
                MessageBox.Show("Error drawing graphic shape: " + ex.Message);
            }
        }

        private void ClearButtonClick(object sender, RoutedEventArgs e)
        {
            // Remove all graphics from the graphics overlay
            this.sketchOverlay.Graphics.Clear();

            // Disable buttons that require graphics
            this.ClearButton.IsEnabled = false;
            this.EditButton.IsEnabled = false;
        }

        private async void EditButtonClick(object sender, RoutedEventArgs e)
        {
            try
            {
                // Allow the user to select a graphic
                Graphic editGraphic = await GetGraphicAsync().ConfigureAwait(true);
                if (editGraphic == null) { return; }

                // Let the user make changes to the graphic's geometry, await the result (updated geometry)
                var newGeometry = await this.MyMapView.SketchEditor.StartAsync(editGraphic.Geometry).ConfigureAwait(true);

                // Display the updated geometry in the graphic
                editGraphic.Geometry = newGeometry;
            }
            catch (TaskCanceledException)
            {
                // Ignore ... let the user cancel editing
            }
            catch (Exception ex)
            {
                // Report exceptions
                MessageBox.Show("Error editing shape: " + ex.Message);
            }
        }
    }
}
